#include <linux/module.h>
#include <linux/kernel.h> /* printk() */
#include <linux/slab.h> /* kmalloc() */
#include <linux/fs.h> /* everything... */
#include <linux/errno.h> /* error codes */
#include <linux/types.h> /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h> /* O_ACCMODE */
#include <asm/system.h> /* cli(), *_flags */
#include <asm/uaccess.h> /* copy_from/to_user */

MODULE_LICENSE("CC4302");

static int hello_load(void);
static void hello_unload(void);
int hello_open(struct inode *inode, struct file *filp);
int hello_close(struct inode *inode, struct file *filp);

int hello_major = 60;
char *hello_buffer;

module_init(hello_load);
module_exit(hello_unload);

struct file_operations hello_fops = { read: NULL,
				      write: NULL,
				      open: hello_open,
				      release: hello_close
};

static int  hello_load(void) {
  int result;

  /* Registering device */
  result = register_chrdev(hello_major, "hello", &hello_fops);

  if (result < 0) {
    printk("Hello: cannot obtain major number %d\n", hello_major);
    return result;
  }

  /* Allocating memory for the buffer */
  hello_buffer = kmalloc(1, GFP_KERNEL);

  if (!hello_buffer) {
    result = -ENOMEM;
    return(result);
  }

  memset(hello_buffer, 0, 1);


  printk("Hello world\n");
  return(0);
}

static void hello_unload(void) {
  unregister_chrdev(hello_major, "hello");

  if (hello_buffer) {
    kfree(hello_buffer);
  }

  printk("Goodby cruel world\n");
}

int hello_open(struct inode *inode, struct file *filp) {
  printk("Opening the device\n");
  return(0);
}

int hello_close(struct inode *inode, struct file *filp) {
  printk("Closing the device\n");
  return(0);
}



